use crate::html::*;
use build_html::{self, Container, ContainerType, Html, HtmlContainer};
use file_service::{FileHandle, FileId, ReadFileError};

pub fn auth(_message: Option<String>) -> build_html::HtmlPage {
    build_html::HtmlPage::new()
        .with_title("Authentication")
        .with_stylesheet("/css")
        .with_container(
            Container::new(ContainerType::Div)
                .with_attributes([("class", "authentication-page")])
                .with_container(
                    Container::new(ContainerType::Div)
                        .with_attributes([("class", "card authentication-form")])
                        .with_html(
                            Form::new()
                                .with_path("/auth")
                                .with_method("post")
                                .with_container(
                                    Container::new(ContainerType::Div)
                                        .with_attributes([("class", "authentication-form__label")])
                                        .with_html(Label::new("for-token-input", "Authentication")),
                                )
                                .with_container(
                                    Container::new(ContainerType::Div)
                                        .with_attributes([("class", "authentication-form__input")])
                                        .with_html(
                                            Input::new("token", "token")
                                                .with_id("for-token-input")
                                                .with_attributes([("size", "50")]),
                                        ),
                                ),
                        ),
                ),
        )
}

pub fn gallery(handles: Vec<Result<FileHandle, ReadFileError>>) -> build_html::HtmlPage {
    let mut page = build_html::HtmlPage::new()
        .with_title("Gallery")
        .with_stylesheet("/css")
        .with_container(
            Container::new(ContainerType::Div)
                .with_attributes([("class", "gallery-page")])
                .with_header(1, "Gallery")
                .with_html(upload_form()),
        );

    let mut gallery = Container::new(ContainerType::Div).with_attributes([("class", "gallery")]);
    for handle in handles {
        let container = match handle {
            Ok(ref handle) => thumbnail(&handle.id).with_html(
                Form::new()
                    .with_path(&format!("/{}", *handle.id))
                    .with_method("post")
                    .with_html(Input::new("hidden", "_method").with_value("delete"))
                    .with_html(Button::new("Delete")),
            ),

            Err(err) => Container::new(ContainerType::Div)
                .with_attributes(vec![("class", "file")])
                .with_paragraph(format!("{:?}", err)),
        };
        gallery.add_container(container);
    }
    page.add_container(gallery);
    page
}

pub fn upload_form() -> Form {
    Form::new()
        .with_path("/upload")
        .with_method("post")
        .with_encoding("multipart/form-data")
        .with_container(
            Container::new(ContainerType::Div)
                .with_attributes([("class", "card upload-form")])
                .with_html(Input::new("file", "file").with_attributes([
                    ("id", "for-selector-input"),
                    ("placeholder", "select file"),
                    ("class", "upload-form__selector"),
                ]))
                .with_html(
                    Button::new("Upload file")
                        .with_attributes([("class", "upload-form__button")])
                        .with_type("submit"),
                ),
        )
}

pub fn thumbnail(id: &FileId) -> Container {
    Container::new(ContainerType::Div)
        .with_attributes(vec![("class", "card thumbnail")])
        .with_html(
            Container::new(ContainerType::Div).with_link(
                format!("/{}", **id),
                Image::new(&format!("{}/tn", **id))
                    .with_attributes([("class", "thumbnail__image")])
                    .to_html_string(),
            ),
        )
}
